package ru.sicamp.sicamphelper.service

import mu.KLogging
import org.springframework.stereotype.Service
import ru.sicamp.sicamphelper.model.data.GroupInput
import ru.sicamp.sicamphelper.model.data.UserInput
import ru.sicamp.sicamphelper.model.request.RequestWrapper
import ru.sicamp.sicamphelper.model.request.schedule.ScheduleGroupRequest
import ru.sicamp.sicamphelper.model.request.schedule.ScheduleRequest
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.model.response.schedule.ScheduleResponse
import ru.sicamp.sicamphelper.repository.ScheduleRepository
import java.time.LocalDate

@Service
class ScheduleService(
    private val scheduleRepository: ScheduleRepository,
) {
    suspend fun getUserSchedule(requestWrapper: RequestWrapper<ScheduleRequest>): ResponseWrapper<ScheduleResponse> {
        val request = requestWrapper.body
        val userInput = request.userInput
        val date = request.date ?: LocalDate.now()
        val schedules = try {
            when (userInput?.type) {
                UserInput.Type.TG_USERNAME -> userInput.tgUsername?.let {
                    scheduleRepository.scheduleByTgUsername(it, date)
                }
                UserInput.Type.GATE_LOGIN -> userInput.gateLogin?.let {
                    scheduleRepository.scheduleByGateLogin(it, date)
                }
                UserInput.Type.USER_ID -> userInput.userId?.let {
                    scheduleRepository.scheduleByUserId(it, date)
                }
                null -> scheduleRepository.scheduleByUser(requestWrapper.issuer, date)
            } ?: return requestWrapper.fail(404, message = "Not found schedules. Bad input parameters")
        } catch (e: Exception) {
            logger.error { e }
            return requestWrapper.fail(500, message = "Error while trying to fetch schedules for user $userInput")
        }

        return requestWrapper.success(ScheduleResponse(schedules))
    }

    suspend fun getSchedulesByGroup(requestWrapper: RequestWrapper<ScheduleGroupRequest>): ResponseWrapper<ScheduleResponse> {
        val request = requestWrapper.body
        val date = request.date ?: LocalDate.now()
        val type = request.group?.type
        val schedules = try {
            when (type) {
                GroupInput.Type.ID -> request.group.id?.let {
                    scheduleRepository.scheduleByGroupId(it, date)
                }
                GroupInput.Type.NAME -> request.group.name?.let {
                    scheduleRepository.scheduleByGroupName(it, date)
                }
                null -> requestWrapper.issuer.group.id.value.let {
                    scheduleRepository.scheduleByGroupId(it, date)
                }
            } ?: return requestWrapper.fail(404, message = "Not found schedules. Bad input parameters")
        } catch (e: Exception) {
            logger.error { e }
            return requestWrapper.fail(500, "Error while trying to fetch schedules for group ${request.group}")
        }

        return requestWrapper.success(ScheduleResponse(schedules))
    }

    companion object : KLogging()
}